@@ -222,7 +222,7 @@ class SharedAnalysisDirectory(AnalysisDirectory):
 
     def get_scratch_directory(self) -> str:
         try:
-            return (
+            return os.path.realpath(
                 subprocess.check_output(["mkscratch", "path", "--subdir", "pyre"])
                 .decode("utf-8")
                 .strip()
@@ -772,6 +772,9 @@ def _get_buck_builder(
                 buck_root=buck_root,
                 buck_mode=buck_mode,
                 output_directory=output_directory,
+                isolation_prefix=(
+                    configuration.get_isolation_prefix_respecting_override()
+                ),
             ),
             [output_directory],
         )
@@ -786,6 +789,7 @@ def _get_buck_builder(
             buck_mode=buck_mode,
             project_name=project_name,
             output_directory=output_directory,
+            isolation_prefix=configuration.get_isolation_prefix_respecting_override(),
         ),
         [output_directory],
     )
