@@ -22,7 +22,7 @@ from ..analysis_directory import AnalysisDirectory, resolve_analysis_directory
 from ..configuration import Configuration, InvalidConfiguration
 from ..exceptions import EnvironmentException
 from ..filesystem import remove_if_exists
-from ..log import StreamLogger
+from ..log import StreamLogger, file_tailer
 from ..process import Process
 from ..socket_connection import SocketConnection, SocketException
 
@@ -92,28 +92,6 @@ class Result:
             )
 
 
-def typeshed_search_path(typeshed_root: str) -> List[str]:
-    search_path = []
-    typeshed_subdirectories = ["stdlib", "third_party"]
-    for typeshed_subdirectory_name in typeshed_subdirectories:
-        typeshed_subdirectory = os.path.join(typeshed_root, typeshed_subdirectory_name)
-        if (
-            not os.path.isdir(typeshed_subdirectory)
-            or typeshed_subdirectory_name == "tests"
-            or typeshed_subdirectory_name[0] == "."
-        ):
-            continue
-
-        # Always prefer newer version over older version
-        version_names = sorted(os.listdir(typeshed_subdirectory), reverse=True)
-        for version_name in version_names:
-            # Anything under 2/ or 2.x is unusable for Pyre
-            if version_name.startswith("2") and version_name != "2and3":
-                continue
-            search_path.append(os.path.join(typeshed_subdirectory, version_name))
-    return search_path
-
-
 def _convert_json_response_to_result(response: json_rpc.Response) -> Result:
     error = response.error
     error_message = None
@@ -383,28 +361,20 @@ class Command(CommandParser, ABC):
             stderr_file = os.path.join(
                 self._configuration.log_directory, "server/server.stdout"
             )
-            with subprocess.Popen(
-                ["tail", "--follow", "--lines=0", stderr_file],
-                stdout=subprocess.PIPE,
-                stderr=subprocess.DEVNULL,
-                universal_newlines=True,
-            ) as stderr_tail:
-                try:
-                    with SocketConnection(
-                        self._configuration.log_directory
-                    ) as socket_connection:
-                        socket_connection.perform_handshake(version_hash)
-                        # pyre-fixme[6]: Expected `Iterable[str]` for 1st param but
-                        #  got `Optional[IO[typing.Any]]`.
-                        with StreamLogger(stderr_tail.stdout):
-                            socket_connection.send(request)
-                            response = socket_connection.read()
-                        result = _convert_json_response_to_result(response)
-                        result.check()
-                        self._socket_result_handler(result)
-                finally:
-                    stderr_tail.terminate()
+            with file_tailer(Path(stderr_file)) as stderr_tail:
+                with SocketConnection(
+                    self._configuration.log_directory
+                ) as socket_connection:
+                    socket_connection.perform_handshake(version_hash)
+                    with StreamLogger(stderr_tail) as stderr_logger:
+                        socket_connection.send(request)
+                        response = socket_connection.read()
+                    result = _convert_json_response_to_result(response)
+                    result.check()
+                    self._socket_result_handler(result)
+            stderr_logger.join()
         except (
+            OSError,
             SocketException,
             ResourceWarning,
             ClientException,
