@@ -17,7 +17,7 @@ from .. import (
 )
 from ..analysis_directory import AnalysisDirectory
 from ..configuration import Configuration
-from .command import ExitCode, IncrementalStyle, Result, State, typeshed_search_path
+from .command import ExitCode, IncrementalStyle, Result, State
 from .reporting import Reporting
 from .start import Start
 
@@ -135,12 +135,10 @@ class Incremental(Reporting):
             ]
         )
 
-        typeshed = self._configuration.get_typeshed_respecting_override()
         search_path = [
             search_path.command_line_argument()
             for search_path in self._configuration.get_existent_search_paths()
-        ] + (typeshed_search_path(typeshed) if typeshed is not None else [])
-        flags.extend(["-source-path", self._analysis_directory.get_root()])
+        ]
         if search_path:
             flags.extend(["-search-path", ",".join(search_path)])
 
