@@ -27,8 +27,8 @@ from .. import command_arguments, log
 from ..analysis_directory import AnalysisDirectory
 from ..annotation_collector import AnnotationCollector
 from ..configuration import Configuration
-from ..error import Error
-from .command import Command, Result, typeshed_search_path
+from ..error import LegacyError
+from .command import Command, Result
 from .reporting import Reporting
 from .statistics import _get_paths, _parse_paths, parse_path_to_module
 
@@ -167,7 +167,7 @@ class Stub:
     stub: Optional[Union[FieldStub, FunctionStub]] = None
 
     def __init__(self, error) -> None:
-        self.path = Path(error.path)
+        self.path = Path(error.error.path)
         self.parent = error.inference.get("parent")
         self.stub = None
         if FunctionStub.is_instance(error.inference):
@@ -234,7 +234,7 @@ class StubFile:
         self._fields = [stub for stub in stubs if stub.is_field()]
         self._functions = [stub for stub in stubs if stub.is_function()]
         self._methods = [stub for stub in stubs if stub.is_method()]
-        self._path = Path(errors[0].path)
+        self._path = Path(errors[0].error.path)
 
     def to_string(self) -> str:
         """We currently ignore nested classes, i.e.:
@@ -297,17 +297,15 @@ class StubFile:
         path.write_text(contents)
 
 
-def generate_stub_files(full_only: bool, errors: Sequence[Error]) -> List[StubFile]:
-    errors = [
-        error
-        for error in errors
-        if error.inference and not (error.is_external_to_global_root())
-    ]
+def generate_stub_files(
+    full_only: bool, errors: Sequence[LegacyError]
+) -> List[StubFile]:
+    errors = [error for error in errors if error.inference]
     files = defaultdict(list)
-    errors.sort(key=lambda error: error.line)
+    errors.sort(key=lambda error: error.error.line)
 
     for error in errors:
-        files[error.path].append(error)
+        files[error.error.path].append(error)
 
     stubs = []
     for _path, errors in files.items():
@@ -422,7 +420,7 @@ def annotate_from_existing_stubs(
 
 def _existing_annotations_as_errors(
     modules: Dict[Path, Optional[libcst.Module]], project_root: str
-) -> List[Error]:
+) -> List[LegacyError]:
     errors = []
     for path_name, module in modules.items():
         path_name = str(path_name)
@@ -432,8 +430,8 @@ def _existing_annotations_as_errors(
             module.visit(collector)
         for stub in collector.stubs:
             errors.append(
-                Error(
-                    error={
+                LegacyError.create(
+                    {
                         "path": path_name.replace(project_root + "/", ""),
                         "inference": stub,
                         "line": 0,
@@ -556,12 +554,10 @@ class Infer(Reporting):
         if len(filter_directories):
             flags.extend(["-filter-directories", ";".join(sorted(filter_directories))])
 
-        typeshed = self._configuration.get_typeshed_respecting_override()
         search_path = [
             search_path.command_line_argument()
             for search_path in self._configuration.get_existent_search_paths()
-        ] + (typeshed_search_path(typeshed) if typeshed is not None else [])
-        flags.extend(["-source-path", self._analysis_directory.get_root()])
+        ]
         if search_path:
             flags.extend(["-search-path", ",".join(search_path)])
 
@@ -571,7 +567,7 @@ class Infer(Reporting):
 
     def _get_errors(
         self, result: Result, bypass_filtering: bool = False, from_stdin: bool = False
-    ) -> Sequence[Error]:
+    ) -> Sequence[LegacyError]:
         analysis_root = os.path.realpath(self._analysis_directory.get_root())
         relative_root = self._original_directory if from_stdin else analysis_root
         errors = self._relativize_errors(relative_root, self._parse_raw_errors(result))
