@@ -4,7 +4,6 @@
 # LICENSE file in the root directory of this source tree.
 
 
-import argparse
 import json
 import logging
 import os
@@ -59,39 +58,6 @@ class Query(Command):
         )
         self.query: str = self._rewrite_paths(query)
 
-    @staticmethod
-    def from_arguments(
-        arguments: argparse.Namespace,
-        original_directory: str,
-        configuration: Configuration,
-        analysis_directory: Optional[AnalysisDirectory] = None,
-    ) -> "Query":
-        return Query(
-            command_arguments.CommandArguments.from_arguments(arguments),
-            original_directory,
-            configuration=configuration,
-            analysis_directory=analysis_directory,
-            query=arguments.query,
-        )
-
-    @classmethod
-    def add_subparser(cls, parser: argparse._SubParsersAction) -> None:
-        query_message = """
-        `https://pyre-check.org/docs/querying-pyre.html` contains examples and
-        documentation for this command, which queries a running pyre server for type,
-        function and attribute information.
-
-        To get a full list of queries, you can run `pyre query help`.
-        """
-        query = parser.add_parser(cls.NAME, epilog=query_message)
-        query.set_defaults(command=cls.from_arguments)
-        query_argument_message = """
-        `pyre query help` will give a full list of available queries for \
-        the running Pyre server.
-         Example: `pyre query "superclasses(int)"`.
-        """
-        query.add_argument("query", help=query_argument_message)
-
     def _flags(self) -> List[str]:
         flags = [self.query]
         flags.extend(["-log-directory", self._configuration.log_directory])
