@@ -9,10 +9,10 @@ import logging
 import os
 from typing import Any, Dict, List, Optional, Sequence, Set
 
-from .. import command_arguments, log
+from .. import command_arguments
 from ..analysis_directory import AnalysisDirectory
 from ..configuration import Configuration
-from ..error import Error
+from ..error import LegacyError, print_errors
 from .command import ClientException, Command, Result
 
 
@@ -33,17 +33,8 @@ class Reporting(Command):
             command_arguments, original_directory, configuration, analysis_directory
         )
 
-    def _print(self, errors: Sequence[Error]) -> None:
-        if errors:
-            length = len(errors)
-            LOG.error("Found %d type error%s!", length, "s" if length > 1 else "")
-        else:
-            LOG.log(log.SUCCESS, "No type errors found")
-
-        if self._output == command_arguments.TEXT:
-            log.stdout.write("\n".join([repr(error) for error in errors]))
-        else:
-            log.stdout.write(json.dumps([error.__dict__ for error in errors]))
+    def _print(self, errors: Sequence[LegacyError]) -> None:
+        print_errors([error.error for error in errors], output=self._output)
 
     def _get_directories_to_analyze(self) -> Set[str]:
         return self._analysis_directory.get_filter_roots()
@@ -68,45 +59,41 @@ class Reporting(Command):
             LOG.debug("Invalid JSON output: %s", json_output)
         return error_list
 
-    def _parse_raw_errors(self, result: Result) -> Sequence[Error]:
+    def _parse_raw_errors(self, result: Result) -> Sequence[LegacyError]:
         result.check()
-        errors: List[Error] = []
+        errors: List[LegacyError] = []
         results: List[Dict[str, Any]] = self._load_errors_from_json(result.output)
         for error in results:
-            errors.append(
-                Error(error, ignore_error=False, external_to_global_root=False)
-            )
+            errors.append(LegacyError.create(error, ignore_error=False))
         return errors
 
     def _relativize_errors(
-        self, relative_root: str, errors: Sequence[Error]
-    ) -> Sequence[Error]:
+        self, relative_root: str, errors: Sequence[LegacyError]
+    ) -> Sequence[LegacyError]:
+        relativized_errors = []
         for error in errors:
-            path = os.path.realpath(os.path.join(relative_root, error.path))
-
-            # Relativize path to user's cwd.
-            relative_path = self._relative_path(path)
-            error.path = relative_path
-
+            path = os.path.realpath(os.path.join(relative_root, error.error.path))
             # Nonexistent paths can be created when search path stubs are renamed.
             if not path.startswith(
                 self._configuration.project_root
             ) or not os.path.exists(path):
-                error.external_to_global_root = True
-        return errors
+                continue
 
-    def _filter_errors(self, errors: Sequence[Error]) -> Sequence[Error]:
-        filtered_errors = [
-            error
-            for error in errors
-            if (not error.is_ignored() and (not (error.is_external_to_global_root())))
-        ]
+            # Relativize path to user's cwd.
+            relative_path = self._relative_path(path)
+            relativized_errors.append(error.with_path(relative_path))
+
+        return relativized_errors
+
+    def _filter_errors(self, errors: Sequence[LegacyError]) -> Sequence[LegacyError]:
+        filtered_errors = [error for error in errors if not error.is_ignored()]
         sorted_errors = sorted(
-            filtered_errors, key=lambda error: (error.path, error.line, error.column)
+            filtered_errors,
+            key=lambda error: (error.error.path, error.error.line, error.error.column),
         )
         return sorted_errors
 
-    def _get_errors(self, result: Result) -> Sequence[Error]:
+    def _get_errors(self, result: Result) -> Sequence[LegacyError]:
         analysis_root = os.path.realpath(self._analysis_directory.get_root())
         errors = self._relativize_errors(analysis_root, self._parse_raw_errors(result))
 
