@@ -3,4 +3,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from . import incremental  # noqa F401
+from . import restart  # noqa F401
 from . import start  # noqa F401
+from . import stop  # noqa F401
