@@ -3,7 +3,9 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import contextlib
 import dataclasses
+import datetime
 import enum
 import json
 import logging
@@ -11,15 +13,27 @@ import os
 import subprocess
 import tempfile
 from pathlib import Path
-from typing import IO, Any, Dict, List, Mapping, Optional, Sequence, Tuple, Union
+from typing import (
+    IO,
+    Any,
+    Dict,
+    Generator,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 from ... import (
     command_arguments,
     commands,
     configuration as configuration_module,
     find_directories,
+    log,
 )
-from . import server_event
+from . import server_connection, server_event
 
 
 LOG: logging.Logger = logging.getLogger(__name__)
@@ -111,7 +125,7 @@ class Arguments:
     store_type_check_resolution: bool = False
     strict: bool = False
     taint_models_path: Sequence[str] = dataclasses.field(default_factory=list)
-    watchman_root: Optional[str] = None
+    watchman_root: Optional[Path] = None
 
     def serialize(self) -> Dict[str, Any]:
         return {
@@ -129,7 +143,7 @@ class Arguments:
             **(
                 {}
                 if self.watchman_root is None
-                else {"watchman_root": self.watchman_root}
+                else {"watchman_root": str(self.watchman_root)}
             ),
             "taint_model_paths": self.taint_models_path,
             "debug": self.debug,
@@ -234,17 +248,19 @@ def create_server_arguments(
     nonexistent directories. It is idempotent though, since it does not alter
     any filesystem state.
     """
-    source_directories = configuration.source_directories or []
+    source_directories: Sequence[str] = configuration.source_directories or []
     if len(source_directories) == 0:
         raise configuration_module.InvalidConfiguration(
             "New server does not have buck support yet."
         )
+    check_directory_allowlist = (
+        list(source_directories)
+        + configuration.get_existent_do_not_ignore_errors_in_paths()
+    )
     return Arguments(
         log_path=configuration.log_directory,
         global_root=configuration.project_root,
-        checked_directory_allowlist=(
-            configuration.get_existent_do_not_ignore_errors_in_paths()
-        ),
+        checked_directory_allowlist=check_directory_allowlist,
         checked_directory_blocklist=(
             configuration.get_existent_ignore_all_errors_paths()
         ),
@@ -255,7 +271,9 @@ def create_server_arguments(
         local_root=configuration.local_root,
         number_of_workers=configuration.get_number_of_workers(),
         parallel=not start_arguments.sequential,
-        saved_state_action=get_saved_state_action(
+        saved_state_action=None
+        if start_arguments.no_saved_state
+        else get_saved_state_action(
             start_arguments, relative_local_root=configuration.relative_local_root
         ),
         search_paths=configuration.get_existent_search_paths(),
@@ -266,10 +284,18 @@ def create_server_arguments(
         taint_models_path=configuration.taint_models_path,
         watchman_root=None
         if start_arguments.no_watchman
-        else str(find_watchman_root(Path(configuration.project_root))),
+        else find_watchman_root(Path(configuration.project_root)),
     )
 
 
+def get_server_identifier(configuration: configuration_module.Configuration) -> str:
+    global_identifier = Path(configuration.project_root).name
+    relative_local_root = configuration.relative_local_root
+    if relative_local_root is None:
+        return global_identifier
+    return f"{global_identifier}/{relative_local_root}"
+
+
 def _write_argument_file(output_file: IO[str], arguments: Arguments) -> None:
     LOG.info(f"Writing server startup configurations into {output_file.name}...")
     serialized_arguments = arguments.serialize()
@@ -305,8 +331,65 @@ def _run_in_foreground(
         return commands.ExitCode.FAILURE
 
 
+class EventParsingException(Exception):
+    pass
+
+
+def parse_server_event(event_string: str) -> server_event.Event:
+    event = server_event.create_from_string(event_string)
+    if event is None:
+        raise EventParsingException(
+            f"Unrecognized status update from server: {event_string}"
+        )
+    elif isinstance(event, server_event.ServerException):
+        raise EventParsingException(f"Failed to start server: {event.message}")
+    return event
+
+
+class BackgroundEventWaiter:
+    wait_on_initialization: bool
+
+    def __init__(self, wait_on_initialization: bool) -> None:
+        self.wait_on_initialization = wait_on_initialization
+
+    def wait_on(self, event_stream: IO[str]) -> None:
+        # The first event is expected to be socket creation
+        initial_event = parse_server_event(event_stream.readline().strip())
+        if isinstance(initial_event, server_event.SocketCreated):
+            if not self.wait_on_initialization:
+                return
+
+            # The second event is expected to be server initialization
+            second_event = parse_server_event(event_stream.readline().strip())
+            if isinstance(second_event, server_event.ServerInitialized):
+                return
+
+        raise EventParsingException(
+            f"Unexpected initial server status update: {initial_event}"
+        )
+
+
+@contextlib.contextmanager
+def background_logging(log_file: Path) -> Generator[None, None, None]:
+    with log.file_tailer(log_file) as log_stream:
+        with log.StreamLogger(log_stream) as logger:
+            yield
+    logger.join()
+
+
+def _create_symbolic_link(source: Path, target: Path) -> None:
+    try:
+        source.unlink()
+    except FileNotFoundError:
+        pass
+    source.symlink_to(target)
+
+
 def _run_in_background(
-    command: Sequence[str], environment: Mapping[str, str], log_directory: Path
+    command: Sequence[str],
+    environment: Mapping[str, str],
+    log_directory: Path,
+    event_waiter: BackgroundEventWaiter,
 ) -> commands.ExitCode:
     # In background mode, we asynchronously start the server with `Popen` and
     # detach it from the current process immediately with `start_new_session`.
@@ -314,7 +397,12 @@ def _run_in_background(
     # Server stderr will be forwarded to dedicated log files.
     # Server stdout will be used as additional communication channel for status
     # updates.
-    with open(str(log_directory / "server.stderr"), "a") as server_stderr:
+    log_file = (
+        log_directory
+        / "new_server"
+        / datetime.datetime.now().strftime("server.stderr.%Y_%m_%d_%H_%M_%S_%f")
+    )
+    with open(str(log_file), "a") as server_stderr:
         server_process = subprocess.Popen(
             command,
             stdout=subprocess.PIPE,
@@ -323,28 +411,43 @@ def _run_in_background(
             start_new_session=True,
             universal_newlines=True,
         )
-        LOG.info("Server is starting in the background.\n")
-        # Block until we receive the first server event from stdout
-        server_stdout = server_process.stdout
-        if server_stdout is None:
-            raise RuntimeError(
-                "subprocess.Popen failed to set up a pipe for server stdout"
-            )
-        else:
-            event_string = server_stdout.readline().strip()
 
-            event = server_event.create_from_string(event_string)
-            if event is None:
-                LOG.error(f"Unrecognized status update from server: {event_string}")
-                return commands.ExitCode.FAILURE
-            elif isinstance(event, server_event.SocketCreated):
-                return commands.ExitCode.SUCCESS
-            elif isinstance(event, server_event.ServerException):
-                LOG.error(f"Failed to start server: {event.message}")
-                return commands.ExitCode.FAILURE
-            else:
-                LOG.warning(f"Unexpected initial server status update: {event}")
-                return commands.ExitCode.FAILURE
+    LOG.info("Server is starting in the background.\n")
+    server_stdout = server_process.stdout
+    if server_stdout is None:
+        raise RuntimeError("subprocess.Popen failed to set up a pipe for server stdout")
+
+    try:
+        # Block until an expected server event is obtained from stdout
+        with background_logging(log_file):
+            event_waiter.wait_on(server_stdout)
+            server_stdout.close()
+
+            # Symlink the log file to a known location for subsequent `pyre incremental`
+            # to find.
+            _create_symbolic_link(
+                log_directory / "new_server" / "server.stderr", log_file
+            )
+
+            return commands.ExitCode.SUCCESS
+    except KeyboardInterrupt:
+        LOG.info("SIGINT received. Terminating background server...")
+
+        # If a background server has spawned and the user hits Ctrl-C, bring down
+        # the spwaned server as well.
+        server_stdout.close()
+        server_process.terminate()
+        server_process.wait()
+
+        # Since we abruptly terminate the background server, it may not have the
+        # chance to clean up the socket file properly. Make sure the file is
+        # removed on our side.
+        try:
+            server_connection.get_default_socket_path(log_directory).unlink()
+        except FileNotFoundError:
+            pass
+
+        raise commands.ClientException("Interrupted by user. No server is spawned.")
 
 
 def run(
@@ -357,17 +460,21 @@ def run(
             "Cannot locate a Pyre binary to run."
         )
 
-    log_directory = Path(configuration.log_directory) / "new_server"
-    log_directory.mkdir(parents=True, exist_ok=True)
+    log_directory = Path(configuration.log_directory)
+    (log_directory / "new_server").mkdir(parents=True, exist_ok=True)
 
-    # Use distinct file name for different PIDs to avoid file write races from
-    # multiple concurrent `pyre start` processes.
+    server_arguments = create_server_arguments(configuration, start_arguments)
+    if not start_arguments.no_watchman and server_arguments.watchman_root is None:
+        LOG.warning(
+            "Cannot find a watchman root. Incremental check will not be functional"
+            " since no filesystem updates will be sent to the Pyre server."
+        )
+
+    LOG.info(f"Starting server at `{get_server_identifier(configuration)}`...")
     with tempfile.NamedTemporaryFile(
         mode="w", prefix="pyre_arguments_", suffix=".json"
     ) as argument_file:
-        _write_argument_file(
-            argument_file, create_server_arguments(configuration, start_arguments)
-        )
+        _write_argument_file(argument_file, server_arguments)
 
         server_command = [binary_location, "newserver", argument_file.name]
         server_environment = {
@@ -381,4 +488,11 @@ def run(
         if start_arguments.terminal:
             return _run_in_foreground(server_command, server_environment)
         else:
-            return _run_in_background(server_command, server_environment, log_directory)
+            return _run_in_background(
+                server_command,
+                server_environment,
+                log_directory,
+                BackgroundEventWaiter(
+                    wait_on_initialization=start_arguments.wait_on_initialization
+                ),
+            )
