@@ -175,6 +175,20 @@ class SitePackageSearchPathElement(SearchPathElement):
         return self
 
 
+def _get_site_roots() -> List[str]:
+    try:
+        return site.getsitepackages()
+    except AttributeError:
+        # There are a few Python versions that ship with a broken venv,
+        # where `getsitepackages` is not available.
+        LOG.warning(
+            "`site.getsitepackages()` is not available in your virtualenv. "
+            "This is a known virtualenv bug and as a workaround please avoid"
+            'using `"site-package"` in your search path configuration.'
+        )
+        return []
+
+
 def create_search_paths(
     json: Union[str, Dict[str, str]], site_roots: Iterable[str]
 ) -> List[SearchPathElement]:
@@ -221,6 +235,7 @@ class PartialConfiguration:
     formatter: Optional[str] = None
     ignore_all_errors: Sequence[str] = field(default_factory=list)
     ignore_infer: Sequence[str] = field(default_factory=list)
+    isolation_prefix: Optional[str] = None
     logger: Optional[str] = None
     number_of_workers: Optional[int] = None
     other_critical_files: Sequence[str] = field(default_factory=list)
@@ -275,6 +290,7 @@ class PartialConfiguration:
             formatter=arguments.formatter,
             ignore_all_errors=[],
             ignore_infer=[],
+            isolation_prefix=arguments.isolation_prefix,
             logger=arguments.logger,
             number_of_workers=None,
             other_critical_files=[],
@@ -355,12 +371,12 @@ class PartialConfiguration:
                     element
                     for json in search_path_json
                     for element in create_search_paths(
-                        json, site_roots=site.getsitepackages()
+                        json, site_roots=_get_site_roots()
                     )
                 ]
             else:
                 search_path = create_search_paths(
-                    search_path_json, site_roots=site.getsitepackages()
+                    search_path_json, site_roots=_get_site_roots()
                 )
 
             partial_configuration = PartialConfiguration(
@@ -388,6 +404,9 @@ class PartialConfiguration:
                     configuration_json, "ignore_all_errors"
                 ),
                 ignore_infer=ensure_string_list(configuration_json, "ignore_infer"),
+                isolation_prefix=ensure_option_type(
+                    configuration_json, "isolation_prefix", str
+                ),
                 logger=ensure_option_type(configuration_json, "logger", str),
                 number_of_workers=ensure_option_type(
                     configuration_json, "workers", int
@@ -483,6 +502,7 @@ class PartialConfiguration:
             ignore_infer=[
                 expand_relative_path(root, path) for path in self.ignore_infer
             ],
+            isolation_prefix=self.isolation_prefix,
             logger=logger,
             number_of_workers=self.number_of_workers,
             other_critical_files=[
@@ -508,8 +528,8 @@ def merge_partial_configurations(
     def overwrite_base(base: Optional[T], override: Optional[T]) -> Optional[T]:
         return base if override is None else override
 
-    def append_base(base: Sequence[T], override: Sequence[T]) -> Sequence[T]:
-        return list(base) + list(override)
+    def prepend_base(base: Sequence[T], override: Sequence[T]) -> Sequence[T]:
+        return list(override) + list(base)
 
     def raise_when_overridden(
         base: Optional[T], override: Optional[T], name: str
@@ -530,35 +550,38 @@ def merge_partial_configurations(
             base.buck_builder_binary, override.buck_builder_binary
         ),
         disabled=overwrite_base(base.disabled, override.disabled),
-        do_not_ignore_all_errors_in=append_base(
+        do_not_ignore_all_errors_in=prepend_base(
             base.do_not_ignore_all_errors_in, override.do_not_ignore_all_errors_in
         ),
         dot_pyre_directory=overwrite_base(
             base.dot_pyre_directory, override.dot_pyre_directory
         ),
-        excludes=append_base(base.excludes, override.excludes),
-        extensions=append_base(base.extensions, override.extensions),
+        excludes=prepend_base(base.excludes, override.excludes),
+        extensions=prepend_base(base.extensions, override.extensions),
         file_hash=overwrite_base(base.file_hash, override.file_hash),
         formatter=overwrite_base(base.formatter, override.formatter),
-        ignore_all_errors=append_base(
+        ignore_all_errors=prepend_base(
             base.ignore_all_errors, override.ignore_all_errors
         ),
-        ignore_infer=append_base(base.ignore_infer, override=override.ignore_infer),
+        ignore_infer=prepend_base(base.ignore_infer, override=override.ignore_infer),
+        isolation_prefix=overwrite_base(
+            base.isolation_prefix, override.isolation_prefix
+        ),
         logger=overwrite_base(base.logger, override.logger),
         number_of_workers=overwrite_base(
             base.number_of_workers, override.number_of_workers
         ),
-        other_critical_files=append_base(
+        other_critical_files=prepend_base(
             base.other_critical_files, override.other_critical_files
         ),
-        search_path=append_base(base.search_path, override.search_path),
+        search_path=prepend_base(base.search_path, override.search_path),
         source_directories=raise_when_overridden(
             base.source_directories,
             override.source_directories,
             name="source_directories",
         ),
         strict=overwrite_base(base.strict, override.strict),
-        taint_models_path=append_base(
+        taint_models_path=prepend_base(
             base.taint_models_path, override.taint_models_path
         ),
         targets=raise_when_overridden(base.targets, override.targets, name="targets"),
@@ -589,6 +612,7 @@ class Configuration:
     formatter: Optional[str] = None
     ignore_all_errors: Sequence[str] = field(default_factory=list)
     ignore_infer: Sequence[str] = field(default_factory=list)
+    isolation_prefix: Optional[str] = None
     logger: Optional[str] = None
     number_of_workers: Optional[int] = None
     other_critical_files: Sequence[str] = field(default_factory=list)
@@ -627,6 +651,7 @@ class Configuration:
             formatter=partial_configuration.formatter,
             ignore_all_errors=partial_configuration.ignore_all_errors,
             ignore_infer=partial_configuration.ignore_infer,
+            isolation_prefix=partial_configuration.isolation_prefix,
             logger=partial_configuration.logger,
             number_of_workers=partial_configuration.number_of_workers,
             other_critical_files=partial_configuration.other_critical_files,
@@ -671,7 +696,20 @@ class Configuration:
                 existent_paths.append(search_path_element)
             else:
                 LOG.debug(f"Filtering out nonexistent search path: {search_path}")
-        return existent_paths
+
+        typeshed_root = self.get_typeshed_respecting_override()
+        typeshed_paths = (
+            []
+            if typeshed_root is None
+            else [
+                SimpleSearchPathElement(str(element))
+                for element in find_directories.find_typeshed_search_paths(
+                    Path(typeshed_root)
+                )
+            ]
+        )
+
+        return existent_paths + typeshed_paths
 
     def get_existent_ignore_infer_paths(self) -> List[str]:
         existent_paths = []
@@ -790,6 +828,15 @@ class Configuration:
                 vaild_extensions.append(extension)
         return vaild_extensions
 
+    def get_isolation_prefix_respecting_override(self) -> Optional[str]:
+        """We need this to disable an isolation prefix set in a configuration.
+        Merely omitting the CLI flag would not disable the isolation prefix
+        because we would just fall back to the configuration value.
+
+        With this, we can pass `--isolation-prefix ''` as a CLI argument or
+        override `isolation_prefix` as `""` in a local configuration."""
+        return None if self.isolation_prefix == "" else self.isolation_prefix
+
 
 def create_configuration(
     arguments: command_arguments.CommandArguments, base_directory: Path
