@@ -4,6 +4,7 @@
 # LICENSE file in the root directory of this source tree.
 
 
+import json
 import logging
 import os
 import shutil
@@ -64,15 +65,36 @@ def _log_statistics(
         )
 
 
+def _show_pyre_version_as_text(
+    binary_version: Optional[str], client_version: str
+) -> None:
+    if binary_version:
+        log.stdout.write(f"Binary version: {binary_version}\n")
+    log.stdout.write(f"Client version: {__version__}\n")
+
+
+def _show_pyre_version_as_json(
+    binary_version: Optional[str], client_version: str
+) -> None:
+    version_json = {
+        **({} if binary_version is None else {"binary": binary_version}),
+        "client": client_version,
+    }
+    log.stdout.write(f"{json.dumps(version_json)}\n")
+
+
 def _show_pyre_version(arguments: command_arguments.CommandArguments) -> None:
+    binary_version: Optional[str] = None
+    client_version: str = __version__
     try:
         configuration = configuration_module.create_configuration(arguments, Path("."))
         binary_version = configuration.get_binary_version()
-        if binary_version:
-            log.stdout.write(f"Binary version: {binary_version}\n")
     except Exception:
         pass
-    log.stdout.write(f"Client version: {__version__}\n")
+    if arguments.output == command_arguments.JSON:
+        _show_pyre_version_as_json(binary_version, client_version)
+    else:
+        _show_pyre_version_as_text(binary_version, client_version)
 
 
 def run_pyre_command(
@@ -96,7 +118,10 @@ def run_pyre_command(
         exit_code = ExitCode.FAILURE
         if isinstance(error, buck.BuckException):
             exit_code = ExitCode.BUCK_ERROR
-    except commands.ClientException as error:
+    except (
+        commands.ClientException,
+        configuration_module.InvalidConfiguration,
+    ) as error:
         client_exception_message = str(error)
         exit_code = ExitCode.FAILURE
     except Exception:
@@ -142,19 +167,43 @@ def _run_incremental_command(
     no_watchman: bool,
 ) -> ExitCode:
     configuration = _create_configuration_with_retry(arguments, Path("."))
-    return run_pyre_command(
-        commands.Incremental(
-            arguments,
-            original_directory=os.getcwd(),
-            configuration=configuration,
-            nonblocking=nonblocking,
-            incremental_style=incremental_style,
-            no_start_server=no_start_server,
+    if arguments.use_command_v2:
+        start_arguments = command_arguments.StartArguments(
+            changed_files_path=arguments.changed_files_path,
+            debug=arguments.debug,
+            load_initial_state_from=arguments.load_initial_state_from,
+            no_saved_state=arguments.no_saved_state,
             no_watchman=no_watchman,
-        ),
-        configuration,
-        arguments.noninteractive,
-    )
+            save_initial_state_to=arguments.save_initial_state_to,
+            saved_state_project=arguments.saved_state_project,
+            sequential=arguments.sequential,
+            show_error_traces=arguments.show_error_traces,
+            store_type_check_resolution=False,
+            terminal=False,
+            wait_on_initialization=True,
+        )
+        return v2.incremental.run(
+            configuration,
+            command_arguments.IncrementalArguments(
+                output=arguments.output,
+                no_start=no_start_server,
+                start_arguments=start_arguments,
+            ),
+        )
+    else:
+        return run_pyre_command(
+            commands.Incremental(
+                arguments,
+                original_directory=os.getcwd(),
+                configuration=configuration,
+                nonblocking=nonblocking,
+                incremental_style=incremental_style,
+                no_start_server=no_start_server,
+                no_watchman=no_watchman,
+            ),
+            configuration,
+            arguments.noninteractive,
+        )
 
 
 def _run_default_command(arguments: command_arguments.CommandArguments) -> ExitCode:
@@ -333,6 +382,7 @@ def _create_configuration_with_retry(
 @click.option("--saved-state-project", type=str, hidden=True)
 @click.option("--features", type=str, hidden=True)
 @click.option("--use-command-v2", is_flag=True, default=False, hidden=True)
+@click.option("--isolation-prefix", type=str, hidden=True)
 def pyre(
     context: click.Context,
     local_configuration: Optional[str],
@@ -369,6 +419,7 @@ def pyre(
     saved_state_project: Optional[str],
     features: Optional[str],
     use_command_v2: bool,
+    isolation_prefix: Optional[str],
 ) -> int:
     arguments = command_arguments.CommandArguments(
         local_configuration=local_configuration,
@@ -407,6 +458,7 @@ def pyre(
         else None,
         features=features,
         use_command_v2=use_command_v2,
+        isolation_prefix=isolation_prefix,
     )
     if arguments.version:
         _show_pyre_version(arguments)
@@ -455,6 +507,12 @@ def pyre(
     default=False,
     help="Provide model query debugging output.",
 )
+@click.option(
+    "--use-cache",
+    is_flag=True,
+    default=False,
+    help="Store information in .pyre/pysa.cache for faster runs.",
+)
 @click.pass_context
 def analyze(
     context: click.Context,
@@ -467,6 +525,7 @@ def analyze(
     rule: Iterable[int],
     find_missing_flows: Optional[str],
     dump_model_query_results: bool,
+    use_cache: bool,
 ) -> int:
     """
     Run Pysa, the inter-procedural static analysis tool.
@@ -492,6 +551,7 @@ def analyze(
                 else None
             ),
             dump_model_query_results=dump_model_query_results,
+            use_cache=use_cache,
         ),
         configuration,
         command_argument.noninteractive,
@@ -548,8 +608,9 @@ def incremental(
     analyzed before returning results. If you'd like to get partial type checking
     results eagerly, you can run `pyre incremental --nonblocking`.
     """
+    command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     return _run_incremental_command(
-        arguments=context.obj["arguments"],
+        arguments=command_argument,
         nonblocking=nonblocking,
         incremental_style=commands.IncrementalStyle.SHALLOW
         if incremental_style == str(commands.IncrementalStyle.SHALLOW)
@@ -847,21 +908,45 @@ def restart(
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    return run_pyre_command(
-        commands.Restart(
-            command_argument,
-            original_directory=os.getcwd(),
-            configuration=configuration,
-            terminal=terminal,
+    if command_argument.use_command_v2:
+        start_arguments = command_arguments.StartArguments(
+            changed_files_path=command_argument.changed_files_path,
+            debug=command_argument.debug,
+            load_initial_state_from=command_argument.load_initial_state_from,
+            no_saved_state=command_argument.no_saved_state,
+            no_watchman=no_watchman,
+            save_initial_state_to=command_argument.save_initial_state_to,
+            saved_state_project=command_argument.saved_state_project,
+            sequential=command_argument.sequential,
+            show_error_traces=command_argument.show_error_traces,
             store_type_check_resolution=store_type_check_resolution,
-            use_watchman=not no_watchman,
-            incremental_style=commands.IncrementalStyle.SHALLOW
-            if incremental_style == str(commands.IncrementalStyle.SHALLOW)
-            else commands.IncrementalStyle.FINE_GRAINED,
-        ),
-        configuration,
-        command_argument.noninteractive,
-    )
+            terminal=terminal,
+            wait_on_initialization=True,
+        )
+        return v2.restart.run(
+            configuration,
+            command_arguments.IncrementalArguments(
+                output=command_argument.output,
+                no_start=False,
+                start_arguments=start_arguments,
+            ),
+        )
+    else:
+        return run_pyre_command(
+            commands.Restart(
+                command_argument,
+                original_directory=os.getcwd(),
+                configuration=configuration,
+                terminal=terminal,
+                store_type_check_resolution=store_type_check_resolution,
+                use_watchman=not no_watchman,
+                incremental_style=commands.IncrementalStyle.SHALLOW
+                if incremental_style == str(commands.IncrementalStyle.SHALLOW)
+                else commands.IncrementalStyle.FINE_GRAINED,
+            ),
+            configuration,
+            command_argument.noninteractive,
+        )
 
 
 @pyre.command()
@@ -920,6 +1005,12 @@ def servers(context: click.Context, subcommand: str) -> int:
     default=str(commands.IncrementalStyle.FINE_GRAINED),
     help="[DEPRECATED] How to approach doing incremental checks.",
 )
+@click.option(
+    "--wait-on-initialization/--no-wait-on-initialization",
+    default=False,
+    hidden=True,
+    help="When `--terminal` is unset, wait for server initialization to finish.",
+)
 @click.pass_context
 def start(
     context: click.Context,
@@ -927,6 +1018,7 @@ def start(
     store_type_check_resolution: bool,
     no_watchman: bool,
     incremental_style: str,
+    wait_on_initialization: bool,
 ) -> int:
     """
     Starts a pyre server as a daemon.
@@ -940,6 +1032,7 @@ def start(
                 changed_files_path=command_argument.changed_files_path,
                 debug=command_argument.debug,
                 load_initial_state_from=command_argument.load_initial_state_from,
+                no_saved_state=command_argument.no_saved_state,
                 no_watchman=no_watchman,
                 save_initial_state_to=command_argument.save_initial_state_to,
                 saved_state_project=command_argument.saved_state_project,
@@ -947,6 +1040,7 @@ def start(
                 show_error_traces=command_argument.show_error_traces,
                 store_type_check_resolution=store_type_check_resolution,
                 terminal=terminal,
+                wait_on_initialization=wait_on_initialization,
             ),
         )
     else:
@@ -1008,15 +1102,18 @@ def stop(context: click.Context) -> int:
     configuration = configuration_module.create_configuration(
         command_argument, Path(".")
     )
-    return run_pyre_command(
-        commands.Stop(
-            command_argument,
-            original_directory=os.getcwd(),
-            configuration=configuration,
-        ),
-        configuration,
-        command_argument.noninteractive,
-    )
+    if command_argument.use_command_v2:
+        return v2.stop.run(configuration)
+    else:
+        return run_pyre_command(
+            commands.Stop(
+                command_argument,
+                original_directory=os.getcwd(),
+                configuration=configuration,
+            ),
+            configuration,
+            command_argument.noninteractive,
+        )
 
 
 # Need the default argument here since this is our entry point in setup.py
@@ -1025,15 +1122,15 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
     with log.configured_logger(noninteractive):
         try:
             return_code = pyre(argv, auto_envvar_prefix="PYRE", standalone_mode=False)
-        except EnvironmentException as error:
-            LOG.error(str(error))
-            return_code = ExitCode.FAILURE
         except configuration_module.InvalidConfiguration as error:
             LOG.error(str(error))
             return ExitCode.CONFIGURATION_ERROR
         except click.ClickException as error:
             error.show()
             return_code = ExitCode.FAILURE
+        except Exception as error:
+            LOG.error(str(error))
+            return_code = ExitCode.FAILURE
     return return_code
 
 
