@@ -7,9 +7,11 @@ import argparse
 import itertools
 import json
 import logging
+import re
 import shutil
 import subprocess
 import sys
+import tempfile
 from itertools import chain
 from pathlib import Path
 from typing import Dict, List, Optional
@@ -25,10 +27,21 @@ class SourceDatabase(TypedDict):
     dependencies: Dict[str, str]
 
 
-def _buck(arguments: List[str]) -> str:
-    return subprocess.check_output(["buck"] + arguments, stderr=subprocess.PIPE).decode(
-        "utf-8"
+def _buck(arguments: List[str], isolation_prefix: Optional[str]) -> str:
+    isolation_prefix_arguments = (
+        ["--isolation_prefix", isolation_prefix] if isolation_prefix is not None else []
     )
+    command = (
+        ["buck"]
+        + isolation_prefix_arguments
+        + arguments
+        + ["--config", "client.id=pyre"]
+    )
+    LOG.debug("Running `%s`", " ".join(command))
+    return subprocess.check_output(
+        command,
+        stderr=subprocess.PIPE,
+    ).decode("utf-8")
 
 
 def _get_buck_query_arguments(
@@ -62,14 +75,36 @@ def _ignore_target(target: str) -> bool:
     return target.endswith(suffixes_for_ignored_targets)
 
 
-def _query_targets(target_specifications: List[str], mode: Optional[str]) -> List[str]:
+def _load_json_ignoring_extra_data(source: str) -> Dict[str, str]:
+    try:
+        return json.loads(source)
+    except json.JSONDecodeError as exception:
+        LOG.debug(f"JSON output: {source}")
+        LOG.warning("Failed to parse JSON. Retrying by ignoring extra data...")
+
+        match = re.search(r"Extra data: line ([0-9]+) column", exception.args[0])
+        if match is None:
+            raise exception
+
+        line_number = int(match.group(1))
+        source_without_extra_data = "\n".join(source.splitlines()[: line_number - 1])
+        return json.loads(source_without_extra_data)
+
+
+def _query_targets(
+    target_specifications: List[str],
+    mode: Optional[str],
+    isolation_prefix: Optional[str],
+) -> List[str]:
     normalized_target_specifications = [
         _normalize_specification(specification)
         for specification in target_specifications
     ]
     query_arguments = _get_buck_query_arguments(normalized_target_specifications, mode)
     LOG.info("Running `buck query`...")
-    specification_targets_dictionary = json.loads(_buck(query_arguments))
+    specification_targets_dictionary = _load_json_ignoring_extra_data(
+        _buck(query_arguments, isolation_prefix)
+    )
     targets = list(chain(*specification_targets_dictionary.values()))
     return [target for target in targets if not _ignore_target(target)]
 
@@ -80,16 +115,23 @@ def _get_buck_build_arguments(targets: List[str]) -> List[str]:
     # onto a consistent set of platforms, but this has a cost of invalidating the
     # parser cache, which may not be worth it.
     return [
-        "build",
         "--show-full-json-output",
         *(f"{target}#source-db" for target in targets),
     ]
 
 
-def _build_targets(targets: List[str]) -> Dict[str, str]:
+def _build_targets(
+    targets: List[str], isolation_prefix: Optional[str]
+) -> Dict[str, str]:
     build_arguments = _get_buck_build_arguments(targets)
     LOG.info("Running `buck build`...")
-    return json.loads(_buck(build_arguments))
+    with tempfile.NamedTemporaryFile(
+        "w+", prefix="pyre_buck_build_arguments"
+    ) as arguments_file:
+        Path(arguments_file.name).write_text("\n".join(build_arguments))
+
+        output = _buck(["build", f"@{arguments_file.name}"], isolation_prefix)
+        return _load_json_ignoring_extra_data(output)
 
 
 def _load_source_databases(
@@ -148,9 +190,14 @@ def build(
     output_directory: Path,
     buck_root: Path,
     mode: Optional[str],
+    isolation_prefix: Optional[str],
 ) -> None:
-    targets = _query_targets(target_specifications, mode)
-    target_path_dictionary = _build_targets(targets)
+    targets = _query_targets(
+        target_specifications,
+        mode,
+        isolation_prefix,
+    )
+    target_path_dictionary = _build_targets(targets, isolation_prefix)
     source_databases = _load_source_databases(target_path_dictionary)
     link_map = _merge_source_databases(source_databases)
     _build_link_tree(link_map, output_directory, buck_root)
@@ -162,6 +209,7 @@ def main(argv: List[str]) -> None:
     parser.add_argument("--debug", action="store_true")
     parser.add_argument("--mode")
     parser.add_argument("--project_name")
+    parser.add_argument("--isolation_prefix")
     parser.add_argument("--output_directory", required=True, type=Path)
     parser.add_argument("--buck_root", dest="buck_root", required=True, type=Path)
     parser.add_argument("target_specifications", nargs="*")
@@ -172,6 +220,7 @@ def main(argv: List[str]) -> None:
         arguments.output_directory,
         arguments.buck_root,
         arguments.mode,
+        arguments.isolation_prefix,
     )
 
 
