@@ -5,7 +5,7 @@
 
 from __future__ import annotations
 
-from typing import Any, List, NamedTuple, Optional, Set, Union
+from typing import Any, List, NamedTuple, Optional, Set
 
 import graphene
 from graphql.execution.base import ResolveInfo
@@ -21,7 +21,7 @@ from ..models import (
     SharedTextKind,
     SourceLocation,
 )
-from . import filters
+from . import filters, run
 
 
 # pyre-fixme[5]: Global expression must be annotated.
@@ -38,20 +38,15 @@ MessageText = aliased(SharedText)
 FeatureText = aliased(SharedText)
 
 
-def _query(info: ResolveInfo) -> "Query":
-    # TODO(T71492980): queries here are run-independent and should be factored out.
-    return Query(info.context["session"], 1)
-
-
 # pyre-ignore[13]: unitialized class attribute
 class IssueQueryResultType(graphene.ObjectType):
-    id: DBID
     concatenated_features: str
 
     class Meta:
         interfaces = (graphene.relay.Node,)
 
     issue_id = graphene.ID()
+    issue_instance_id = graphene.ID()
 
     code = graphene.Int()
     message = graphene.String()
@@ -67,23 +62,26 @@ class IssueQueryResultType(graphene.ObjectType):
     sink_names = graphene.List(graphene.String)
     features = graphene.List(graphene.String)
 
+    is_new_issue = graphene.Boolean()
+
     min_trace_length_to_sources = graphene.Int()
     min_trace_length_to_sinks = graphene.Int()
 
-    def resolve_issue_id(self, info: ResolveInfo) -> DBID:
-        return self.id
-
     def resolve_sources(self, info: ResolveInfo) -> List[str]:
-        return list(_query(info).sources(self.id))
+        # pyre-ignore[6]: graphene too dynamic.
+        return list(sources(info.context["session"], self.issue_instance_id))
 
     def resolve_source_names(self, info: ResolveInfo) -> List[str]:
-        return list(_query(info).source_names(self.id))
+        # pyre-ignore[6]: graphene too dynamic.
+        return list(source_names(info.context["session"], self.issue_instance_id))
 
     def resolve_sinks(self, info: ResolveInfo) -> List[str]:
-        return list(_query(info).sinks(self.id))
+        # pyre-ignore[6]: graphene too dynamic.
+        return list(sinks(info.context["session"], self.issue_instance_id))
 
     def resolve_sink_names(self, info: ResolveInfo) -> List[str]:
-        return list(_query(info).sink_names(self.id))
+        # pyre-ignore[6]: graphene too dynamic.
+        return list(sink_names(info.context["session"], self.issue_instance_id))
 
     def resolve_features(self, info: ResolveInfo) -> List[str]:
         # pyre-ignore[6]: graphene too dynamic.
@@ -91,7 +89,8 @@ class IssueQueryResultType(graphene.ObjectType):
 
 
 class IssueQueryResult(NamedTuple):
-    id: DBID
+    issue_id: DBID
+    issue_instance_id: DBID
 
     code: int
     message: str
@@ -101,6 +100,8 @@ class IssueQueryResult(NamedTuple):
     filename: str
     location: SourceLocation
 
+    is_new_issue: bool
+
     min_trace_length_to_sources: int
     min_trace_length_to_sinks: int
 
@@ -110,12 +111,14 @@ class IssueQueryResult(NamedTuple):
     # pyre-fixme[2]: Parameter annotation cannot be `Any`.
     def from_record(record: Any) -> IssueQueryResult:
         return IssueQueryResult(
-            id=record.id,
+            issue_id=record.issue_id,
+            issue_instance_id=record.issue_instance_id,
             code=record.code,
             message=record.message,
             callable=record.callable,
             filename=record.filename,
             location=record.location,
+            is_new_issue=record.is_new_issue,
             min_trace_length_to_sources=record.min_trace_length_to_sources,
             min_trace_length_to_sinks=record.min_trace_length_to_sinks,
             features=set(record.concatenated_features.split(","))
@@ -124,12 +127,11 @@ class IssueQueryResult(NamedTuple):
         )
 
 
-class Query:
-    # pyre-fixme[3]: Return type must be annotated.
-    def __init__(self, session: Session, run_id: Union[DBID, int]):
+class Instance:
+    def __init__(self, session: Session, run_id: Optional[DBID] = None) -> None:
         self._session: Session = session
         self._predicates: List[filters.Predicate] = []
-        self._run_id = run_id
+        self._run_id: DBID = run_id or run.latest(session)
 
     def get(self) -> List[IssueQueryResult]:
         features = (
@@ -156,12 +158,15 @@ class Query:
         )
         query = (
             self._session.query(
-                IssueInstance.id,
+                IssueInstance.id.label("issue_instance_id"),
                 FilenameText.contents.label("filename"),
                 IssueInstance.location,
+                # pyre-ignore[16]: SQLAlchemy
+                Issue.id.label("issue_id"),
                 Issue.code,
                 CallableText.contents.label("callable"),
                 MessageText.contents.label("message"),
+                IssueInstance.is_new_issue,
                 IssueInstance.min_trace_length_to_sources,
                 IssueInstance.min_trace_length_to_sinks,
                 features.c.concatenated_features,
@@ -194,27 +199,32 @@ class Query:
 
         return issues
 
-    def where(self, *predicates: filters.Predicate) -> "Query":
+    def where(self, *predicates: filters.Predicate) -> "Instance":
         self._predicates.extend(predicates)
         return self
 
-    def where_issue_id_is(self, issue_id: Optional[int]) -> "Query":
+    def where_issue_instance_id_is(self, issue_id: Optional[int]) -> "Instance":
         if issue_id is not None:
             self._predicates.append(filters.Equals(IssueInstance.id, issue_id))
         return self
 
-    def where_codes_is_any_of(self, codes: List[int]) -> "Query":
+    def where_is_new_issue(self, is_new_issue: Optional[bool]) -> "Instance":
+        if is_new_issue:
+            self._predicates.append(filters.Equals(IssueInstance.is_new_issue, True))
+        return self
+
+    def where_codes_is_any_of(self, codes: List[int]) -> "Instance":
         return self.where(filters.Like(Issue.code, codes))
 
-    def where_callables_is_any_of(self, callables: List[str]) -> "Query":
+    def where_callables_is_any_of(self, callables: List[str]) -> "Instance":
         return self.where(filters.Like(CallableText.contents, callables))
 
-    def where_path_is_any_of(self, paths: List[str]) -> "Query":
+    def where_path_is_any_of(self, paths: List[str]) -> "Instance":
         return self.where(filters.Like(FilenameText.contents, paths))
 
     def where_trace_length_to_sinks(
         self, minimum: Optional[int] = None, maximum: Optional[int] = None
-    ) -> "Query":
+    ) -> "Instance":
         return self.where(
             filters.InRange(
                 IssueInstance.min_trace_length_to_sinks, lower=minimum, upper=maximum
@@ -223,70 +233,62 @@ class Query:
 
     def where_trace_length_to_sources(
         self, minimum: Optional[int] = None, maximum: Optional[int] = None
-    ) -> "Query":
+    ) -> "Instance":
         return self.where(
             filters.InRange(
                 IssueInstance.min_trace_length_to_sources, lower=minimum, upper=maximum
             )
         )
 
-    def where_any_features(self, features: List[str]) -> "Query":
+    def where_any_features(self, features: List[str]) -> "Instance":
         return self.where(filters.HasAny(set(features)))
 
-    def where_all_features(self, features: List[str]) -> "Query":
+    def where_all_features(self, features: List[str]) -> "Instance":
         return self.where(filters.HasAll(set(features)))
 
-    def where_exclude_features(self, features: List[str]) -> "Query":
+    def where_exclude_features(self, features: List[str]) -> "Instance":
         return self.where(filters.HasNone(set(features)))
 
-    def sources(self, issue_id: DBID) -> Set[str]:
-        return self.get_leaves_issue_instance(
-            self._session, int(issue_id), SharedTextKind.SOURCE
+
+def sources(session: Session, issue_id: DBID) -> Set[str]:
+    return _get_leaves(session, issue_id, SharedTextKind.SOURCE)
+
+
+def source_names(session: Session, issue_id: DBID) -> Set[str]:
+    return _get_leaves(session, issue_id, SharedTextKind.SOURCE_DETAIL)
+
+
+def sinks(session: Session, issue_id: DBID) -> Set[str]:
+    return _get_leaves(session, issue_id, SharedTextKind.SINK)
+
+
+def sink_names(session: Session, issue_id: DBID) -> Set[str]:
+    return _get_leaves(session, issue_id, SharedTextKind.SINK_DETAIL)
+
+
+def features(session: Session, issue_id: DBID) -> Set[str]:
+    return _get_leaves(session, issue_id, SharedTextKind.FEATURE)
+
+
+def _get_leaves(
+    session: Session, issue_instance_id: DBID, kind: SharedTextKind
+) -> Set[str]:
+    message_ids = [
+        int(id)
+        for id, in session.query(SharedText.id)
+        .distinct(SharedText.id)
+        .join(
+            IssueInstanceSharedTextAssoc,
+            SharedText.id == IssueInstanceSharedTextAssoc.shared_text_id,
         )
+        .filter(IssueInstanceSharedTextAssoc.issue_instance_id == issue_instance_id)
+        .filter(SharedText.kind == kind)
+    ]
 
-    def source_names(self, issue_id: DBID) -> Set[str]:
-        return self.get_leaves_issue_instance(
-            self._session, int(issue_id), SharedTextKind.SOURCE_DETAIL
+    leaf_lookup = {
+        int(id): contents
+        for id, contents in session.query(SharedText.id, SharedText.contents).filter(
+            SharedText.kind == kind
         )
-
-    def sinks(self, issue_id: DBID) -> Set[str]:
-        return self.get_leaves_issue_instance(
-            self._session, int(issue_id), SharedTextKind.SINK
-        )
-
-    def sink_names(self, issue_id: DBID) -> Set[str]:
-        return self.get_leaves_issue_instance(
-            self._session, int(issue_id), SharedTextKind.SINK_DETAIL
-        )
-
-    def features(self, issue_id: DBID) -> Set[str]:
-        return self.get_leaves_issue_instance(
-            self._session, int(issue_id), SharedTextKind.FEATURE
-        )
-
-    def get_leaves_issue_instance(
-        self, session: Session, issue_instance_id: int, kind: SharedTextKind
-    ) -> Set[str]:
-        message_ids = [
-            int(id)
-            for id, in session.query(SharedText.id)
-            .distinct(SharedText.id)
-            .join(
-                IssueInstanceSharedTextAssoc,
-                SharedText.id == IssueInstanceSharedTextAssoc.shared_text_id,
-            )
-            .filter(IssueInstanceSharedTextAssoc.issue_instance_id == issue_instance_id)
-            .filter(SharedText.kind == kind)
-        ]
-        return self._leaf_dict_lookups(message_ids, kind, session)
-
-    def _leaf_dict_lookups(
-        self, message_ids: List[int], kind: SharedTextKind, session: Session
-    ) -> Set[str]:
-        leaf_dict = {
-            int(id): contents
-            for id, contents in session.query(
-                SharedText.id, SharedText.contents
-            ).filter(SharedText.kind == kind)
-        }
-        return {leaf_dict[id] for id in message_ids if id in leaf_dict}
+    }
+    return {leaf_lookup[id] for id in message_ids if id in leaf_lookup}
