@@ -77,6 +77,6 @@ class FixmeSingle(ErrorSuppressingCommand):
         local_root = configuration.get_directory().resolve()
         title = "{} for {}".format(
             "Update pyre version" if self._upgrade_version else "Suppress pyre errors",
-            str(local_root.relative_to(project_configuration.resolve())),
+            str(local_root.relative_to(project_configuration.parent.resolve())),
         )
         self._repository.commit_changes(commit=(not self._no_commit), title=title)
