@@ -13,7 +13,7 @@ import libcst
 from typing_extensions import Final
 
 from ..configuration import Configuration
-from ..errors import Errors, PartialErrorSuppression
+from ..errors import Errors
 from ..filesystem import (
     LocalMode,
     add_local_mode,
@@ -71,7 +71,7 @@ class TargetsToConfiguration(ErrorSuppressingCommand):
     ) -> None:
         super().__init__(command_arguments, repository)
         self._subdirectory: Final[Optional[str]] = subdirectory
-        self._glob: int = glob
+        self._glob_threshold: Optional[int] = glob
         self._fixme_threshold: int = fixme_threshold
         self._pyre_only: bool = pyre_only
         self._strict: bool = strict
@@ -123,7 +123,7 @@ class TargetsToConfiguration(ErrorSuppressingCommand):
 
     def remove_target_typing_fields(self, files: List[Path]) -> None:
         LOG.info("Removing typing options from %s targets files", len(files))
-        if self._pyre_only and not self._glob:
+        if self._pyre_only and not self._glob_threshold:
             for path in files:
                 targets_file = Path(path)
                 source = targets_file.read_text()
@@ -143,35 +143,9 @@ class TargetsToConfiguration(ErrorSuppressingCommand):
             ] + [str(file) for file in files]
             subprocess.run(remove_typing_fields_command)
 
-    def convert_directory(self, directory: Path) -> None:
-        all_targets = find_targets(directory, pyre_only=self._pyre_only)
-        if not all_targets:
-            LOG.warning("No configuration created because no targets found.")
-            return
-        if self._glob:
-            new_targets = ["//" + str(directory) + "/..."]
-            targets_files = [
-                directory / path
-                for path in get_filesystem().list(
-                    str(directory), patterns=[r"**/TARGETS"]
-                )
-            ]
-        else:
-            new_targets = []
-            targets_files = []
-            for path, targets in all_targets.items():
-                targets_files.append(Path(path))
-                new_targets += [
-                    "//" + path.replace("/TARGETS", "") + ":" + target.name
-                    for target in targets
-                ]
-
-        apply_strict = self._strict and any(
-            target.strict
-            for target in [
-                target for target_list in all_targets.values() for target in target_list
-            ]
-        )
+    def find_or_create_configuration(
+        self, directory: Path, new_targets: List[str]
+    ) -> Configuration:
         configuration_path = directory / ".pyre_configuration.local"
         if configuration_path.exists():
             LOG.warning(
@@ -191,29 +165,76 @@ class TargetsToConfiguration(ErrorSuppressingCommand):
 
             # Add newly created configuration files to version control
             self._repository.add_paths([configuration_path])
+        return configuration
 
-        # Remove all type-related target settings
-        self.remove_target_typing_fields(targets_files)
-        if not self._pyre_only:
-            remove_non_pyre_ignores(directory)
+    def convert_directory(self, directory: Path) -> None:
+        all_targets = find_targets(directory, pyre_only=self._pyre_only)
+        if not all_targets:
+            LOG.warning("No configuration created because no targets found.")
+            return
 
-        all_errors = configuration.get_errors()
-        error_threshold = self._fixme_threshold
-        glob_threshold = self._glob
+        # Set strict default to true if any binary or unittest targets are strict.
+        apply_strict = self._strict and any(
+            target.strict
+            for target in [
+                target for target_list in all_targets.values() for target in target_list
+            ]
+        )
 
-        for path, errors in all_errors.paths_to_errors.items():
-            errors = list(errors)
-            error_count = len(errors)
-            if glob_threshold and error_count > glob_threshold:
+        # Collect targets.
+        new_targets = []
+        targets_files = []
+        for path, targets in all_targets.items():
+            targets_files.append(Path(path))
+            new_targets += [
+                "//" + path.replace("/TARGETS", "") + ":" + target.name
+                for target in targets
+            ]
+
+        configuration = self.find_or_create_configuration(directory, new_targets)
+
+        # Try setting a glob target.
+        glob_threshold = self._glob_threshold
+        all_errors = None
+        if glob_threshold is not None:
+            original_targets = configuration.targets
+            configuration.targets = ["//" + str(directory) + "/..."]
+            configuration.write()
+
+            all_errors = configuration.get_errors()
+            if any(
+                len(errors) > glob_threshold
+                for errors in all_errors.paths_to_errors.values()
+            ):
                 # Fall back to non-glob codemod.
                 LOG.info(
                     "Exceeding error threshold of %d; falling back to listing "
                     "individual targets.",
                     glob_threshold,
                 )
-                self._repository.revert_all(remove_untracked=True)
-                self._glob = None
-                return self.run()
+                configuration.targets = original_targets
+                configuration.write()
+                all_errors = configuration.get_errors()
+            else:
+                targets_files = [
+                    directory / path
+                    for path in get_filesystem().list(
+                        str(directory), patterns=[r"**/TARGETS"]
+                    )
+                ]
+        if not all_errors:
+            all_errors = configuration.get_errors()
+
+        # Remove all type-related target settings.
+        self.remove_target_typing_fields(targets_files)
+        if not self._pyre_only:
+            remove_non_pyre_ignores(directory)
+
+        # Suppress errors in individual files where fixme threshold is not exceeded.
+        error_threshold = self._fixme_threshold
+        for path, errors in all_errors.paths_to_errors.items():
+            errors = list(errors)
+            error_count = len(errors)
             if error_threshold and error_count > error_threshold:
                 LOG.info(
                     "%d errors found in `%s`. Adding file-level ignore.",
@@ -222,70 +243,33 @@ class TargetsToConfiguration(ErrorSuppressingCommand):
                 )
                 add_local_mode(path, LocalMode.IGNORE)
             else:
-                try:
-                    self._apply_suppressions(Errors(errors))
-                except PartialErrorSuppression:
-                    LOG.warning(f"Could not suppress all errors in {path}")
-                    LOG.info("Run with --unsafe to force suppression anyway.")
-                    self._repository.revert_all(remove_untracked=True)
+                self._apply_suppressions(Errors(errors))
 
+        # Spin up strict codemod if applicable, otherwise skip to final clean and lint.
         if apply_strict:
             LOG.info(
                 "Some targets were running strict type checking. "
                 "Adding strict setting to configuration."
             )
             strict_codemod = StrictDefault(
-                self._command_arguments,
+                command_arguments=CommandArguments(
+                    comment=self._comment,
+                    max_line_length=self._max_line_length,
+                    truncate=self._truncate,
+                    unsafe=self._unsafe,
+                    force_format_unsuppressed=self._force_format_unsuppressed,
+                    lint=self._lint,
+                    no_commit=True,
+                    should_clean=True,
+                ),
                 repository=self._repository,
                 local_configuration=directory,
                 remove_strict_headers=True,
                 fixme_threshold=0,
             )
             strict_codemod.run()
-
-        # Lint and re-run pyre once to resolve most formatting issues
-        if self._lint:
-            if self._repository.format():
-                errors = configuration.get_errors(should_clean=False)
-                try:
-                    self._apply_suppressions(errors)
-                except PartialErrorSuppression:
-                    LOG.warning(f"Could not suppress all errors in {path}")
-                    LOG.info("Run with --unsafe to force suppression anyway.")
-                    self._repository.revert_all(remove_untracked=True)
-
-    def run(self) -> None:
-        # TODO(T62926437): Basic integration testing.
-        subdirectory = self._subdirectory
-        subdirectory = Path(subdirectory) if subdirectory else Path.cwd()
-        LOG.info(
-            "Converting typecheck targets to pyre configurations in `%s`", subdirectory
-        )
-        configuration_directories = self._gather_directories(subdirectory)
-        converted = []
-        for directory in configuration_directories:
-            if all(
-                str(directory).startswith(str(converted_directory)) is False
-                for converted_directory in converted
-            ):
-                self.convert_directory(directory)
-                converted.append(directory)
-
-        summary = self._repository.MIGRATION_SUMMARY
-        glob = self._glob
-        if glob:
-            summary += (
-                f"\n\nConfiguration target automatically expanded to include "
-                f"all subtargets, expanding type coverage while introducing "
-                f"no more than {glob} fixmes per file."
-            )
-        title = f"Convert type check targets in {subdirectory} to use configuration"
-        self._repository.commit_changes(
-            commit=(not self._no_commit),
-            title=title,
-            summary=summary,
-            set_dependencies=False,
-        )
+        else:
+            self._suppress_errors(configuration)
 
     def _gather_directories(self, subdirectory: Path) -> List[Path]:
         configurations = find_files(subdirectory, ".pyre_configuration.local")
@@ -318,3 +302,36 @@ class TargetsToConfiguration(ErrorSuppressingCommand):
                 current_depth += 1
             configuration_directories.extend(missing_directories)
         return [Path(directory) for directory in configuration_directories]
+
+    def run(self) -> None:
+        # TODO(T62926437): Basic integration testing.
+        subdirectory = self._subdirectory
+        subdirectory = Path(subdirectory) if subdirectory else Path.cwd()
+        LOG.info(
+            "Converting typecheck targets to pyre configurations in `%s`", subdirectory
+        )
+        configuration_directories = self._gather_directories(subdirectory)
+        converted = []
+        for directory in configuration_directories:
+            if all(
+                str(directory).startswith(str(converted_directory)) is False
+                for converted_directory in converted
+            ):
+                self.convert_directory(directory)
+                converted.append(directory)
+
+        summary = self._repository.MIGRATION_SUMMARY
+        glob = self._glob_threshold
+        if glob:
+            summary += (
+                f"\n\nConfiguration target automatically expanded to include "
+                f"all subtargets, expanding type coverage while introducing "
+                f"no more than {glob} fixmes per file."
+            )
+        title = f"Convert type check targets in {subdirectory} to use configuration"
+        self._repository.commit_changes(
+            commit=(not self._no_commit),
+            title=title,
+            summary=summary,
+            set_dependencies=False,
+        )
